/*
Copyright (c) 1997-2005, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.tmf;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;


/**Counts the bytes it passes upstream.
 * While we are blocked reading input we temporarily decrement
 * the connection count on the grounds that we are I/O bound
 * rather than internally resource bound.
 */
public final class ThroughputMonitorRequestStream extends ServletInputStream
    {
    /**Construct a new filter to monitor input flow.
     * Package visible to allow construction from the ThroughputMonitorFilter.
     */
    ThroughputMonitorRequestStream(ThroughputMonitorFilterBase f,
                                          HttpServletRequest response)
        throws IOException
        {
        super();
        stream = response.getInputStream();
        filter = f;
        }

    /**The original output stream... */
    private final ServletInputStream stream;

    /**The throughput filter... */
    private final ThroughputMonitorFilterBase filter;


    /**Read a whole array of bytes. */
    public int read(byte b[]) throws IOException
        { return(read(b, 0, b.length)); }

    /**Read a single byte; hideously inefficient. */
    public int read() throws IOException
        {
        final boolean didDecrement = filter.adjustBusyConnectionCount(false);
        try { return(stream.read()); }
        finally { if(didDecrement) { filter.adjustBusyConnectionCount(true); } }
        }

    public int read(byte b[], int off, int len) throws IOException
        {
        final boolean didDecrement = filter.adjustBusyConnectionCount(false);
        try { return(super.read(b, off, len)); }
        finally { if(didDecrement) { filter.adjustBusyConnectionCount(true); } }
        }
    }
